@@ -24,7 +24,8 @@ import traceback
 
 from .errors import *
 from random import choice, choices
-class FindPil:
+
+class _FindPil:
     """Not important for you, just makes things easier for me on the back-end ;)"""
     found: bool = True
 
@@ -32,7 +33,8 @@ _ascii_lowercase = "abcdef"
 _incorrect_lowercase = "ghijklmnopqrstuvwxyz"
 _digits = "0123456789"
 
-options = all_options = ["dataUri", "flip", "rotate", "scale", "radius", "size", "backgroundColor", "translateX", "translateY"]
+options = all_options = ["dataUri", "flip", "rotate", "scale", "radius", "size", "backgroundColor", "translateX",
+                         "translateY"]
 styles = ["adventurer", "adventurer-neutral", "avataaars", "big-ears", "big-ears-neutral",
           "big-smile", "bottts", "croodles", "croodles-neutral", "identicon", "initials",
           "micah", "miniavs", "open-peeps", "personas", "pixel-art", "pixel-art-neutral"]
@@ -43,6 +45,7 @@ class DColor:
     """
     Base class for DAvatar's colors.
     """
+
     def __init__(self, html_code: str = "#ffffff"):
         """
         Colors used in this package. This uses HTML color codes!
@@ -52,7 +55,7 @@ class DColor:
         :raise dicebear.errors.IncorrectColor:
         """
         if html_code in ["random", "rnd"]:
-            html_code = '#' + ''.join(choices(_ascii_lowercase+_digits, k=6))
+            html_code = '#' + ''.join(choices(_ascii_lowercase + _digits, k=6))
         if not html_code.startswith("#"):
             html_code = "#" + html_code
         if len(html_code) not in [4, 7] or any(x in html_code for x in _incorrect_lowercase):
@@ -61,10 +64,13 @@ class DColor:
 
     def __str__(self):
         return f"{self.html_code}"
+
     def __repr__(self):
         return f"{self.html_code}"
+
     def __eq__(self, other):
         return self.html_code == other.html_code
+
     def __ne__(self, other):
         return self.html_code != other.html_code
 
@@ -132,6 +138,7 @@ class DFormat:
     list = all_formats = ["png", "svg"]
     png = "png"
     svg = "svg"
+
     def __init__(self):
         """Only use `.attribute` to use a format."""
         pass
@@ -148,7 +155,8 @@ class DFormat:
 
 
 default_options: dict = {options[0]: False, options[1]: False, options[2]: 0, options[3]: 100, options[4]: 0,
-             options[5]: 0, options[6]: DColor(), options[7]: 0, options[8]: 0}
+                         options[5]: 0, options[6]: DColor(), options[7]: 0, options[8]: 0}
+
 
 class DOptions(dict):
     """
@@ -156,8 +164,10 @@ class DOptions(dict):
     """
     empty: dict = {}
     default_options = default = default_options
+
     def __init__(self, *, dataUri: bool = False, flip: bool = False, rotate: int = 0, scale: int = 100,
-                 radius: int = 0, size: int = 0, backgroundColor: DColor = DColor(), translateX: int = 0, translateY: int = 0, **kwargs):
+                 radius: int = 0, size: int = 0, backgroundColor: DColor = DColor(), translateX: int = 0,
+                 translateY: int = 0, **kwargs):
         """
         Go to https://github.com/jvherck/dicebear#base-options to see all info (important for minimum and maximum values for each option!)
 
@@ -165,13 +175,14 @@ class DOptions(dict):
         """
         # kwargs: list = [dataUri, flip, rotate, scale, radius, size, backgroundColor, translateX, translateY]
         dic = kwargs.get("fromdict", {})
-        current: dict = {"dataUri": dataUri, "flip": flip, "rotate": rotate, "scale": scale, "radius": radius, "size": size,
-                     "backgroundColor": backgroundColor, "translateX": translateX, "translateY": translateY}
+        current: dict = {"dataUri": dataUri, "flip": flip, "rotate": rotate, "scale": scale, "radius": radius,
+                         "size": size,
+                         "backgroundColor": backgroundColor, "translateX": translateX, "translateY": translateY}
         if dic:
             for item in dic:
                 if item not in default_options.keys() or dic[item] == default_options[item]:
                     dic.pop(item, None)
-        else: # if not dic
+        else:  # if not dic
             for item in current:
                 if item in default_options and current[item] != default_options[item]:
                     dic.update({item: current[item]})
@@ -184,12 +195,13 @@ class DOptions(dict):
         return type(self)
 
 
-
 def pilcheck(func):
     """Decorator to check if package Pillow is installed"""
+
     def wrapper(*args, **kwargs):
-        if FindPil.found is True:
+        if _FindPil.found is True:
             return func(*args, **kwargs)
         else:
             log_error(PILError())
+
     return wrapper
